// COPYRIGHT LICENSE:  This information contains sample code provided in source code form.
// You may copy, modify, and distribute these sample programs in any form without payment
// to IBM for the purposes of developing, using, marketing or distributing application
// programs conforming to the application programming interface for the operating platform
// for which the sample code is written. Notwithstanding anything to the contrary,
// IBM PROVIDES THE SAMPLE SOURCE CODE ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES,
// EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS
// OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND
// ANY WARRANTY OR CONDITION OF NON-INFRINGEMENT.  IBM SHALL NOT BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
// OPERATION OF THE SAMPLE SOURCE CODE.  IBM HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE SAMPLE SOURCE CODE.
//
// (C) Copyright IBM Corp. 2011, 2012.
// All Rights Reserved. Licensed Materials - Property of IBM.

package com.ibm.ws.batch.ilc.sample;

import java.util.Properties;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.Set;
import java.util.HashSet;
import java.math.BigDecimal;

import com.ibm.websphere.batch.BatchConstants;
import com.ibm.websphere.batch.BatchJobStepInterface;

import com.ibm.websphere.batch.ilc.ILContainerFactory;
import com.ibm.websphere.batch.ilc.ILContainer;
import com.ibm.websphere.batch.ilc.ILContainerException;
import com.ibm.websphere.batch.ilc.ILProcedureException;

import com.ibm.ws.batch.ilc.sample.NATLTEST.NATLTEST;
import com.ibm.ws.batch.ilc.sample.PRIMITVE.PRIMITVE;
import com.ibm.ws.batch.ilc.sample.HELLO1.HELLO1;
import com.ibm.ws.batch.ilc.sample.HELLO2.HELLO2;
import com.ibm.ws.batch.ilc.sample.SIMPLE.SIMPLE;
import com.ibm.ws.batch.ilc.sample.PRIMITVE.parameters.PRIMITVE_DFHCOMMAREA;
import com.ibm.ws.batch.ilc.sample.PRIMITVE.parameters.PRIMITVE_RetDecBigDec;
import com.ibm.ws.batch.ilc.sample.NATLTEST.parameters.NATLTEST_DFHCOMMAREA;

public class ILCDataTypes implements BatchJobStepInterface
{
  private static final String _cName = ILCDataTypes.class.getName();
  private static Logger logger = Logger.getLogger(_cName);

  public static final String COBOL_PROCS_TO_INVOKE = "cobol_procs_to_invoke";
  private Properties _props = null;
  private int _maxRecordCount = 0;

  private int _recordI = 0;

  private ILContainer _ilc = null;
  private NATLTEST _natltest = null;
  private PRIMITVE _primitve = null;
  private HELLO1 _hello1 = null;
  private HELLO2 _hello2 = null;
  private SIMPLE _simple = null;

  private Set _cobolProcsToInvoke = null;

  public void createJobStep()
  {
    final String mName = "createJobStep";
    logger.entering(_cName,mName);

    _maxRecordCount = Integer.parseInt(_props.getProperty(ILCEmpProcessor.MAX_RECORD_COUNT,"10"));
    String cobolProcs = _props.getProperty(COBOL_PROCS_TO_INVOKE);

    logger.fine(ILCEmpProcessor.MAX_RECORD_COUNT + ": " + _maxRecordCount);
    logger.fine(COBOL_PROCS_TO_INVOKE + ": " + cobolProcs);

    if (cobolProcs != null && cobolProcs.length() > 0)
    {
      String[] cobolProcsArr = cobolProcs.split("[:;,./]"); // whatever delimiter.
      _cobolProcsToInvoke = new HashSet();
      for (int i=0; i < cobolProcsArr.length; ++i)
        _cobolProcsToInvoke.add(cobolProcsArr[i]);
    }

    try
    {
      // Create the Java-COBOL ILContainer
      _ilc = ILContainerFactory.getFactory().create();

      // Create the COBOL PROCEDURE stub
      _natltest = new NATLTEST();
      _primitve = new PRIMITVE();
      _hello1 = new HELLO1();
      _hello2 = new HELLO2();
      _simple = new SIMPLE();
    }
    catch (ILContainerException ilce)
    {
      logger.log(Level.FINE,"Caught ILContainerException",ilce);
      throw new RuntimeException(ilce);
    }
    catch (Throwable t)
    {
      logger.log(Level.FINE,"Caught Throwable",t);
      throw new RuntimeException(t);
    }
  }

  public int destroyJobStep()
  {
    final String mName = "createJobStep";
    logger.entering(_cName,mName);
    logger.exiting(_cName,mName);
    return 0;
  }

  public Properties getProperties()
  {
    return _props;
  }

  public int processJobStep()
  {
    final String mName = "processJobStep";
    _recordI++;
    logger.entering(_cName,mName,"record count: " + _recordI);
    int rc = BatchConstants.STEP_CONTINUE;

    try
    {
      // Invoke COBOL.

      if (_cobolProcsToInvoke == null || _cobolProcsToInvoke.contains("NATLTEST"))
      {
        logger.fine("Invoking NATLTEST...");
        _ilc.invokeProcedure(_natltest);
        printNATLTEST();
      }

      if (_cobolProcsToInvoke == null || _cobolProcsToInvoke.contains("PRIMITVE"))
      {
        if (_recordI == 1)
          initPRIMITVE();   // Initialize parm data

        logger.fine("Invoking PRIMITVE...");
        _ilc.invokeProcedure(_primitve);
        printPRIMITVE();
      }

      if (_cobolProcsToInvoke == null || _cobolProcsToInvoke.contains("HELLO1"))
      {
        logger.fine("Invoking HELLO1...");
        _ilc.invokeProcedure(_hello1);
        logger.fine("HELLO1.getReturnCode = " + _hello1.getReturnCode());
      }

      if (_cobolProcsToInvoke == null || _cobolProcsToInvoke.contains("HELLO2"))
      {
        logger.fine("Invoking HELLO2...");
        _ilc.invokeProcedure(_hello2);
      }

      if (_cobolProcsToInvoke == null || _cobolProcsToInvoke.contains("SIMPLE"))
      {
        logger.fine("Invoking SIMPLE");
        _ilc.invokeProcedure(_simple);
        logger.fine("SIMPLE.getReturnCode = " + _simple.getReturnCode());
      }
    }
    catch (ILProcedureException ilpe)
    {
      logger.log(Level.FINE,"Caught ILProcedureException",ilpe);
      return BatchConstants.STEP_COMPLETE_EXECUTION_FAILED;
    }
    catch (ILContainerException ilce)
    {
      logger.log(Level.FINE,"Caught ILContainerException",ilce);
      return BatchConstants.STEP_COMPLETE_EXECUTION_FAILED;
    }

    if (_recordI >= _maxRecordCount)
      rc = BatchConstants.STEP_COMPLETE;
    else
      rc = BatchConstants.STEP_CONTINUE;

    logger.exiting(_cName,mName,"rc = " + rc);
    return rc;
  }

  public void setProperties(Properties props)
  {
    _props = props;
  }


  /**
   * Initialize the parameters for the PRIMITVE program.
   */
  private void initPRIMITVE()
  {
    final String mName = "initPRIMITVE";
    logger.entering(_cName,mName);

    PRIMITVE_DFHCOMMAREA p = _primitve.getPRIMITVE_DFHCOMMAREA();
    p.setUsZoneDec(BigDecimal.valueOf(100.0));
    p.setSZoneDec(BigDecimal.valueOf(-100.0));
    p.setZoneDecSL(BigDecimal.valueOf(-100.0));
    p.setZoneDecSLS(BigDecimal.valueOf(-100.0));
    p.setZoneDecSTS(BigDecimal.valueOf(-100.0));
    p.setUsPackDec(BigDecimal.valueOf(100.0));
    p.setSPackDec(BigDecimal.valueOf(-100.0));
    p.setUsBinary((short)100);
    p.setSBinary((short)-100);
    p.setSBinaryDec(BigDecimal.valueOf(-100.0));
    p.setInterFloat((float)100.0);
    p.setInterDouble(100.0);
    p.setExternFP("+99.99E+00");
    p.setSingleChar("A");
    p.setPICXStr("hello cobol");
    p.setPICAStr("from java");
    p.setDecIntArray(new int[] { 10, 20, 30, 40, 50 });

    logger.exiting(_cName,mName);
  }

  private void printPRIMITVE()
  {
    PRIMITVE_DFHCOMMAREA p = _primitve.getPRIMITVE_DFHCOMMAREA();
    logger.fine("PRIMITVE_DFHCOMMAREA raw: " + p);
    logger.fine("UsZoneDec=" + p.getUsZoneDec());
    logger.fine("SZoneDec=" + p.getSZoneDec());
    logger.fine("ZoneDecSL=" + p.getZoneDecSL());
    logger.fine("ZoneDecSLS=" + p.getZoneDecSLS());
    logger.fine("ZoneDecSTS=" + p.getZoneDecSTS());
    logger.fine("UsPackDec=" + p.getUsPackDec());
    logger.fine("SPackDec=" + p.getSPackDec());
    logger.fine("UsBinary=" + p.getUsBinary());
    logger.fine("SBinary=" + p.getSBinary());
    logger.fine("SBinaryDec=" + p.getSBinaryDec());
    logger.fine("InterFloat=" + p.getInterFloat());
    logger.fine("InterDouble=" + p.getInterDouble());
    logger.fine("ExternFP=" + p.getExternFP());
    logger.fine("SingleChar=" + p.getSingleChar());
    logger.fine("PICXStr=" + p.getPICXStr());
    logger.fine("PICAStr=" + p.getPICAStr());
    logger.fine("DecByte=" + p.getDecByte());
    logger.fine("DecShort=" + p.getDecShort());
    logger.fine("DecInt=" + p.getDecInt());
    logger.fine("DecLong=" + p.getDecLong());
    logger.fine("DecBigDec=" + p.getDecBigDec());
    int[] a = p.getDecIntArray();
    for (int i=0; i < a.length; ++i)
      logger.fine("DecIntArray[" + i + "]=" + a[i]);
    PRIMITVE_RetDecBigDec p_ret = _primitve.getReturnValue();
    logger.fine("RetDecBigDec=" + p_ret.getRetDecBigDec());
  }

  private void printNATLTEST()
  {
    NATLTEST_DFHCOMMAREA n = _natltest.getNATLTEST_DFHCOMMAREA();
    logger.fine("NATLTEST_DFHCOMMAREA raw: " + n);
    logger.fine("PicN=" + n.getPicN());
    logger.fine("NEditted=" + n.getNEditted());
    logger.fine("NNumEditted=" + n.getNNumEditted());
    logger.fine("NExtFloat=" + n.getNExtFloat());
    logger.fine("UsExtDec=" + n.getUsExtDec());
    logger.fine("ExtDecSLS=" + n.getExtDecSLS());
    logger.fine("ExtDecSTS=" + n.getExtDecSTS());
    logger.fine("NATLTEST RETURN-CODE=" + _natltest.getReturnCode());
  }


}
