// COPYRIGHT LICENSE:  This information contains sample code provided in source code form.
// You may copy, modify, and distribute these sample programs in any form without payment
// to IBM for the purposes of developing, using, marketing or distributing application
// programs conforming to the application programming interface for the operating platform
// for which the sample code is written. Notwithstanding anything to the contrary,
// IBM PROVIDES THE SAMPLE SOURCE CODE ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES,
// EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS
// OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND
// ANY WARRANTY OR CONDITION OF NON-INFRINGEMENT.  IBM SHALL NOT BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
// OPERATION OF THE SAMPLE SOURCE CODE.  IBM HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE SAMPLE SOURCE CODE.
//
// (C) Copyright IBM Corp. 2011, 2012.
// All Rights Reserved. Licensed Materials - Property of IBM.

package com.ibm.ws.batch.ilc.sample;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.math.BigDecimal;

import com.ibm.ws.batch.ilc.sample.parameters.ILCEMPREC;

public class ILCEmpRecord
{
  // The fieldMap is read from the first record of the input file.
  // It maps the field names to the field indices for the file input.
  private static String _inputFieldMapStr = "EMPNO|FIRSTNME|LASTNAME|EDLEVEL|SALARY";
  private static String[] _inputFieldMap = _inputFieldMapStr.split("\\|");

  private final static String _outputFieldMapStr = "EMPNO|FIRSTNME|LASTNAME|EDLEVEL|SALARY";
  private final static String[] _outputFieldMap = _outputFieldMapStr.split("\\|");

  private short _empno;
  private String _firstnme;
  private String _lastname;
  private short _edlevel;
  private BigDecimal _salary;

  public ILCEmpRecord(ResultSet resultSet)
  {
    try
    {
      _empno = resultSet.getShort("EMPNO");
      _firstnme = resultSet.getString("FIRSTNME");
      _lastname = resultSet.getString("LASTNAME");
      _edlevel = resultSet.getShort("EDLEVEL");
      _salary = resultSet.getBigDecimal("SALARY");
    }
    catch (SQLException e)
    {
      throw new RuntimeException(e);
    }
  }


  public ILCEmpRecord(String record)
  {
    // ILC Emp fields delimited by '|'.
    // First record (header) of input file maps the field names.

    try
    {
      String[] fields = record.split("\\|");

      _empno = Short.parseShort(fields[getFieldIdx("EMPNO")]);
      _firstnme = fields[getFieldIdx("FIRSTNME")];
      _lastname = fields[getFieldIdx("LASTNAME")];
      _edlevel = Short.parseShort(fields[getFieldIdx("EDLEVEL")]);
      _salary = new BigDecimal(fields[getFieldIdx("SALARY")]);
    }
    catch (NumberFormatException e)
    {
      throw new RuntimeException(e);
    }
  }


  public ILCEmpRecord(ILCEMPREC ilcemprec)
  {
    readFromILCEMPREC(ilcemprec);
  }


  public short getEMPNO()
  {
    return _empno;
  }

  public void setEMPNO(short empno)
  {
    _empno = empno;
  }

  public String getFIRSTNME()
  {
    return _firstnme;
  }

  public void setFIRSTNME(String firstnme)
  {
    _firstnme = firstnme;
  }

  public String getLASTNAME()
  {
    return _lastname;
  }

  public void setLASTNAME(String lastname)
  {
    _lastname = lastname;
  }

  public short getEDLEVEL()
  {
    return _edlevel;
  }

  public void setEDLEVEL(short edlevel)
  {
    _edlevel = edlevel;
  }

  public BigDecimal getSALARY()
  {
    return _salary;
  }

  public void setSALARY(BigDecimal salary)
  {
    _salary = salary;
  }


  public String toString()
  {
    return "ILCEmpRecord:" + marshalRecord(); 
  }

  public String marshalRecord()
  {
    return "" + _empno + "|" + _firstnme + "|" + _lastname + "|" + _edlevel + "|" + _salary;
  }


  public int getFieldIdx(String fieldName)
  {
    for (int i=0; i < _inputFieldMap.length; ++i)
    {
      if (fieldName.equalsIgnoreCase(_inputFieldMap[i]))
        return i;
    }
    throw new RuntimeException("Couldn't find fieldName " + fieldName + " in fieldMap " + _inputFieldMapStr);
  }


  public static void setInputFieldMap(String fieldMapStr)
  {
    _inputFieldMapStr = fieldMapStr;
    _inputFieldMap = _inputFieldMapStr.split("\\|");
  }

  public static String[] getInputFieldMap()
  {
    return _inputFieldMap;
  }

  public static String getOutputFieldMapStr()
  {
    return _outputFieldMapStr;
  }

  public ILCEMPREC writeToILCEMPREC(ILCEMPREC ilcemprec)
  {
    ilcemprec.setEmpno(getEMPNO());
    ilcemprec.setFirstnme__len((short)getFIRSTNME().length());
    ilcemprec.setFirstnme__name(getFIRSTNME());
    ilcemprec.setLastname__len((short)getLASTNAME().length());
    ilcemprec.setLastname__name(getLASTNAME());
    ilcemprec.setEdlevel(getEDLEVEL());
    ilcemprec.setSalary(getSALARY());
    return ilcemprec;
  }

  public void readFromILCEMPREC(ILCEMPREC ilcemprec)
  {
    setEMPNO(ilcemprec.getEmpno());
    setFIRSTNME(ilcemprec.getFirstnme__name().substring(0,ilcemprec.getFirstnme__len()));
    setLASTNAME(ilcemprec.getLastname__name().substring(0,ilcemprec.getLastname__len()));
    setEDLEVEL(ilcemprec.getEdlevel());
    setSALARY(ilcemprec.getSalary());
  }

}
