// COPYRIGHT LICENSE:  This information contains sample code provided in source code form.
// You may copy, modify, and distribute these sample programs in any form without payment
// to IBM for the purposes of developing, using, marketing or distributing application
// programs conforming to the application programming interface for the operating platform
// for which the sample code is written. Notwithstanding anything to the contrary,
// IBM PROVIDES THE SAMPLE SOURCE CODE ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES,
// EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS
// OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND
// ANY WARRANTY OR CONDITION OF NON-INFRINGEMENT.  IBM SHALL NOT BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
// OPERATION OF THE SAMPLE SOURCE CODE.  IBM HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE SAMPLE SOURCE CODE.
//
// (C) Copyright IBM Corp. 2011, 2012.
// All Rights Reserved. Licensed Materials - Property of IBM.

package com.ibm.ws.batch.ilc.sample.bds;

import java.sql.ResultSet;
import java.util.Properties;
import java.util.logging.Logger;

import com.ibm.websphere.batch.devframework.datastreams.patternadapter.JDBCReaderPattern;
import com.ibm.ws.batch.ilc.sample.ILCEmpRecord;

public class ILCEmpJdbcReader implements JDBCReaderPattern
{
  private static final String _cName = ILCEmpJdbcReader.class.getName();
  private static Logger logger = Logger.getLogger(_cName);

  private Properties _props = null;
  private int _lastRecordID = 0;

  public Object fetchRecord(ResultSet resultSet)
  {
    ILCEmpRecord ilcEmpRecord = new ILCEmpRecord(resultSet);

    _lastRecordID = (int) ilcEmpRecord.getEMPNO();

    return ilcEmpRecord;
  }

  public String getInitialLookupQuery()
  {
    String sql = _props.getProperty("initial.sql");

    if (sql == null || sql.length() == 0)
      sql = "SELECT EMPNO,FIRSTNME,LASTNAME,EDLEVEL,SALARY FROM ILCSCHEMA.EMP ORDER BY EMPNO";

    logger.fine("getInitialLookupQuery: " + sql);

    return sql;
  }

  public String getRestartQuery(String restartToken)
  {
    _lastRecordID = Integer.parseInt(restartToken);
    String sql = _props.getProperty("restart.sql");

    if (sql == null || sql.length() == 0)
      sql =  "SELECT EMPNO,FIRSTNME,LASTNAME,EDLEVEL,SALARY FROM ILCSCHEMA.EMP WHERE EMPNO > " + _lastRecordID + " ORDER BY EMPNO";
    else
      sql.replace("<restartToken>",restartToken);

    logger.fine("getRestartQuery: " + sql);
    return sql;
  }

  public String getRestartTokens()
  {
    String restartToken = Integer.toString(_lastRecordID);
    logger.fine("getRestartTokens: " + restartToken);
    return restartToken;
  }

  public void initialize(Properties props)
  {
    _props = props;
  }

}
